@@ -18,7 +18,7 @@ are:
 
 - `s:string` disables all further preprocessing and interprets `string` as an UTF8 encoded string
 - `u:string` same, but as an UTF16-LE encoded string
-- `h:string` assumes that `string` is a hexadecimal string of even length and returns the decoded byte sequence.
+- `h:string` assumes that `string` is a hexadecimal string and returns the decoded byte sequence.
 - any unit's name can be prefixed to the string, i.e. `esc:\\n` corresponds to the line break character.
 
 If a multibin argument does not use any handler, refinery first interprets the string as the path
@@ -73,27 +73,41 @@ second example, the expression
 
 will be parsed as the hexadecimal representation of the SHA256 hash of the file `foobar.txt`.
 """
+from __future__ import annotations
+
 import ast
+import itertools
+import inspect
 
-from itertools import cycle, count, chain
 from argparse import ArgumentTypeError
 from contextlib import suppress
-from functools import update_wrapper
-from typing import Optional, Tuple, Union, Mapping, Any, List, TypeVar, Iterable, ByteString, Callable
+from functools import update_wrapper, reduce, lru_cache
+from typing import AnyStr, Optional, Tuple, Union, Mapping, Any, List, TypeVar, Iterable, ByteString, Callable
+from typing import get_type_hints
 
 from ..lib.frame import Chunk
 from ..lib.tools import isbuffer
 from ..lib.loader import resolve, EntryNotFound
 
-FinalType = TypeVar('T')
+FinalType = TypeVar('FinalType')
+
 DelayedType = Callable[[ByteString], FinalType]
-MaybeDelayedType = Union[DelayedType, FinalType]
+MaybeDelayedType = Union[DelayedType[FinalType], FinalType]
 
 
 class ParserError(ArgumentTypeError): pass
 class VariablesMissing(ParserError): pass
 
 
+class RepeatedInteger(int):
+    """
+    This class serves as a dual-purpose result for `refinery.lib.argformats.numseq`
+    types. It is an integer, but can be infinitely iterated.
+    """
+    def __iter__(self):
+        while True: yield self
+
+
 class PythonExpression:
     """
     Implements a parser for any Python expression with a prescribed set of variable
@@ -148,7 +162,10 @@ class PythonExpression:
         ast.USub
     }
 
-    def __init__(self, definition, *variables, constants=None, all_variables_allowed=False):
+    def __init__(self, definition: AnyStr, *variables, constants=None, all_variables_allowed=False):
+        self.definition = definition
+        if not isinstance(definition, str):
+            definition = definition.decode('utf8')
         constants = constants or {}
         variables = set(variables) | set(constants)
         try:
@@ -176,7 +193,6 @@ class PythonExpression:
 
         self.variables = names
         self.constants = constants
-        self.definition = definition
 
     def __str__(self):
         return self.definition
@@ -185,7 +201,7 @@ class PythonExpression:
         values.update(self.constants)
         for v in self.variables:
             if v not in values:
-                raise ValueError(F'unbound variable {v}, cannot evaluate')
+                raise VariableMissing(v)
         return eval(self.definition, None, values)
 
     @classmethod
@@ -228,7 +244,7 @@ def utf8(x):
     return x.encode('UTF8')
 
 
-class IncompatibleHandler(ValueError):
+class IncompatibleHandler(ArgumentTypeError):
     """
     This exception is generated when `refinery.lib.argformats.DelayedArgument` handlers
     are chained in an incompatible way.
@@ -253,7 +269,7 @@ class TooLazy(Exception):
     pass
 
 
-class VariableMissing(RuntimeError):
+class VariableMissing(ArgumentTypeError):
     def __init__(self, name):
         super().__init__(F'The variable {name} is not defined.')
         self.name = name
@@ -284,7 +300,21 @@ class DelayedArgumentDispatch:
         def __call__(self, *args, **kwargs):
             return self.ego(self.arg, *args, **kwargs)
 
-    def __init__(self, method):
+        def __getattr__(self, key):
+            return getattr(self.ego, key)
+
+    @classmethod
+    def Inherit(cls, parent: DelayedArgument):
+        def wrap(method):
+            dispatcher = cls(method)
+            parent_dispatcher = parent.handler
+            dispatcher.handlers.update(parent_dispatcher.handlers)
+            dispatcher.final.update(parent_dispatcher.final)
+            dispatcher.units = parent_dispatcher.units
+            return dispatcher
+        return wrap
+
+    def __init__(self, method, parent: Optional[DelayedArgumentDispatch] = None):
         update_wrapper(self, method)
         self.default = method
         self.handlers = {}
@@ -311,6 +341,10 @@ class DelayedArgumentDispatch:
     def __call__(self, instance, data, modifier=None, *args):
         try:
             handler = self.default if modifier is None else self.handlers[modifier]
+            name = next(itertools.islice(inspect.signature(handler).parameters.values(), 1, None)).name
+            hint = get_type_hints(handler).get(name, None)
+            if hint == Iterable[type(data)]:
+                data = (data,)
             return handler(instance, data, *args)
         except KeyError:
             unit = self._get_unit(modifier, *args)
@@ -340,6 +374,24 @@ class DelayedArgumentDispatch:
         return _register
 
 
+def LazyPythonExpression(expression: str) -> MaybeDelayedType[Any]:
+    parser = PythonExpression(expression, all_variables_allowed=True)
+    if parser.variables:
+        def evaluate(data: Chunk):
+            try:
+                return parser(**data.meta)
+            except VariableMissing:
+                # It is possible that a byte string can accidentally look like a valid Python
+                # expression, e.g.: B0fGtH*9/HKlwT:
+                definition = parser.definition
+                if isinstance(definition, str):
+                    definition = definition.encode('utf8')
+                return definition
+        return evaluate
+    else:
+        return parser()
+
+
 class DelayedArgument(LazyEvaluation):
     """
     This base class for delayed argument parsers implements parsing
@@ -350,6 +402,7 @@ class DelayedArgument(LazyEvaluation):
     _ARG_SPLIT_TOKEN = ','
 
     def __init__(self, expression: str):
+        self.expression = expression
         self.modifiers = []
         self.finalized = False
         while not self.finalized:
@@ -402,11 +455,11 @@ class DelayedArgument(LazyEvaluation):
                 return name, arguments, expression[k + 1:]
         return None, (), expression
 
-    def __call__(self, data: Union[ByteString, Chunk, None] = None) -> bytes:
+    def __call__(self, data: Optional[ByteString] = None) -> bytes:
         arg = self.seed
         mod = iter(self.modifiers)
         if not self.finalized:
-            mod = chain(((None, ()),), mod)
+            mod = itertools.chain(((None, ()),), mod)
         for name, arguments in mod:
             if isbuffer(arg):
                 arg = Chunk(arg)
@@ -426,23 +479,15 @@ class DelayedArgument(LazyEvaluation):
                 arg = arg(data)
         return arg
 
-    def handler(self, expression: str):
-        """
-        This method is overwritten by children of `refinery.lib.argformats.DelayedArgument`
-        to implement the default handler.
-        """
-        raise NotImplementedError
-
-
-class DelayedBinaryArgument(DelayedArgument):
+    def default_handler(self, expression: str) -> bytes:
+        try:
+            return open(expression, 'rb').read()
+        except Exception:
+            return utf8(expression)
 
     @DelayedArgumentDispatch
-    def handler(self, expr: str) -> bytes:
-        try:
-            return open(expr, 'rb').read()
-        except Exception:
-            pass
-        return utf8(expr)
+    def handler(self, expression: str) -> bytes:
+        return self.default_handler(expression)
 
     @handler.register('s', final=True)
     def s(self, string: str) -> bytes:
@@ -507,8 +552,6 @@ class DelayedBinaryArgument(DelayedArgument):
         as a `refinery.lib.argformats.sliceobj`. The result contains the corresponding slice
         of the input data.
         """
-        if not region or region.lower() == 'all':
-            region = ':'
         return lambda d: memoryview(d)[sliceobj(region, **d.meta)]
 
     @handler.register('x', 'cut', final=True)
@@ -529,11 +572,15 @@ class DelayedBinaryArgument(DelayedArgument):
         return extract
 
     def _interpret_variable(self, name: str, obj: Any):
-        if isbuffer(obj) or isinstance(obj, (list, tuple, set, int)):
+        if isbuffer(obj) or isinstance(obj, int):
             return obj
         if isinstance(obj, str):
             return utf8(obj)
-        raise ValueError(F'The meta variable {name} is of type {type(obj).__name__} and no conversion to bytes is known.')
+        if isinstance(obj, (tuple, set, frozenset)):
+            obj = list(obj)
+        if isinstance(obj, list):
+            return obj
+        raise ArgumentTypeError(F'The meta variable {name} is of type {type(obj).__name__} and no conversion is known.')
 
     @handler.register('var', final=True)
     def var(self, name: str) -> bytes:
@@ -544,8 +591,8 @@ class DelayedBinaryArgument(DelayedArgument):
         def extract(data: Chunk):
             try:
                 result = data.meta[name]
-            except KeyError as K:
-                raise VariableMissing(name) from K
+            except KeyError:
+                raise VariableMissing(name)
             return self._interpret_variable(name, result)
         return extract
 
@@ -564,58 +611,8 @@ class DelayedBinaryArgument(DelayedArgument):
             return self._interpret_variable(name, result)
         return extract
 
-
-def multibin(expression: Union[str, bytes, bytearray]) -> Union[bytes, DelayedBinaryArgument]:
-    """
-    This is the argument parser type that uses `refinery.lib.argformats.DelayedBinaryArgument`.
-    """
-    if not isinstance(expression, str):
-        return bytes(expression)
-    arg = DelayedBinaryArgument(expression)
-    with suppress(TooLazy):
-        return arg()
-    return arg
-
-
-class DelayedNumSeqArgument(DelayedArgument):
-    """
-    A parser for sequences of numeric arguments. As `refinery.lib.argformats.DelayedNumSeqArgument.handler`
-    uses `refinery.lib.argformats.multibin`, it is possible to use any handler specified in
-    `refinery.lib.argformats.DelayedBinaryArgument` as long as these handlers precede any of the handlers
-    defined here.
-    """
-
-    def _mbin(self, expr: str) -> bytes:
-        binary = multibin(expr)
-        if not binary:
-            raise ArgumentTypeError('received empty binary argument')
-        return binary
-
-    def _iter(self, unknown):
-        if hasattr(unknown, '__iter__'):
-            it = list(unknown)
-            if all(isinstance(t, int) for t in it):
-                return it
-        if isinstance(unknown, int):
-            return (unknown,)
-        raise ArgumentTypeError(
-            F'numseq parser encountered {unknown} of type {type(unknown).__name__}, '
-            F'but only integers are supported.'
-        )
-
-    @DelayedArgumentDispatch
-    def handler(self, expression: str) -> Iterable[int]:
-        """
-        The default handler: Attempts to parse the input expression as a sequence of integers
-        and uses `refinery.lib.argformats.multibin` to parse it if that fails.
-        """
-        try:
-            return self._iter(PythonExpression.evaluate(expression))
-        except Exception:
-            return self._mbin(expression)
-
     @handler.register('e', 'eval')
-    def ev(self, expression: Union[str, ByteString]) -> Iterable[int]:
+    def eval(self, expression: Union[str, ByteString]) -> Any:
         """
         Final modifier `e:expression` or `eval:expression`; uses a `refinery.lib.argformats.PythonExpression`
         parser to process expressions. The expression can contain any meta variable that is attached to the
@@ -627,71 +624,247 @@ class DelayedNumSeqArgument(DelayedArgument):
                 expression = expression.decode('ascii')
             except AttributeError:
                 return expression
-        evaluator = PythonExpression(expression, all_variables_allowed=True)
-        if evaluator.variables:
-            def finalize(chunk):
-                return self._iter(evaluator(**chunk.meta))
-            return finalize
-        return self._iter(evaluator())
+        return LazyPythonExpression(expression)
 
-    @handler.register('unpack', final=True)
-    def unpack(self, expression: str, size=None) -> Iterable[int]:
+    @handler.register('btoi')
+    def btoi(self, binary: ByteString, size=None) -> Iterable[int]:
         """
-        Final modifier `unpack[size]:expression`; uses `refinery.lib.chunks.unpack` to
-        convert a sequence of bytes into a sequence of numbers by unpacking them. The `expression`
-        parameter is parsed with `refinery.lib.argformats.multibin` yielding this byte string.
-        The optional parameter `size` has to be an integer expression whose absolute value gives
-        the size of each encoded number in bytes. Its default value is `0`, which corresponds to
-        choosing the size automatically in the following manner: If the length of the buffer is
-        uneven, the value `1` is chosen. If the length modulo `4` is nonzero, the value `2` is
-        chosen. If the length is divisible by `4`, then `4` is chosen.
+        The modifier `btoi[size=0]:data` uses `refinery.lib.chunks.unpack` to convert a sequence
+        of bytes into a sequence of integers by unpacking them. The `expression` parameter is parsed
+        with `refinery.lib.argformats.multibin` yielding this byte string. The optional parameter
+        `size` has to be an integer expression whose absolute value gives the size of each encoded
+        number in bytes. Its default value is `0`, which corresponds to choosing the size
+        automatically in the following manner: If the length of the buffer is uneven, the value `1`
+        is chosen. If the length modulo `4` is nonzero, the value `2` is chosen. If the length is
+        divisible by `4`, then `4` is chosen. To unpack as big endian as opposed to the default
+        little endian, a negative `size` has to be specified. The absolute value of `size` will be
+        used.
         """
-        from .chunks import unpack
+        from . import chunks
         size = int(size, 0) if size else 0
-        obig = size < 0
+        bigE = size < 0
         size = abs(size)
-        mbin = self._mbin(expression)
-        if not callable(mbin) and size:
-            return list(unpack(mbin, size, obig))
-        def delayed(d): # noqa
-            data = mbin(d) if callable(mbin) else mbin
-            size = {1: 1, 2: 2, 3: 1, 0: 4}[len(data) % 4]
-            return list(unpack(data, size, False))
-        return delayed
+        if not size:
+            n = len(binary)
+            if n % 2:
+                size = 1
+            elif n % 4:
+                size = 2
+            else:
+                size = 4
+        return list(chunks.unpack(binary, size, bigE))
+
+    @handler.register('itob')
+    def itob(self, integers: Iterable[int], size=None) -> ByteString:
+        """
+        The modifier `itob[size=0]:integers` is the inverse of `btoi` and works in the same way,
+        except that the case `size=0` is handled in the following way: The handler inspects all
+        integers in the input and determines the minimum block size required to pack all of them.
+        """
+        from . import chunks
+        size = int(size, 0) if size else 0
+        bigE = size < 0
+        size = abs(size)
+        if not size:
+            def byte_length(n: int):
+                width, overflow = divmod(n.bit_length(), 8)
+                if overflow: width += 1
+                return width
+            if not isinstance(integers, list):
+                integers = list(integers)
+            size = max(byte_length(n) for n in integers)
+            size = max(size, 1)
+        return chunks.pack(integers, size, bigE)
 
     @handler.register('inc')
     def inc(self, it: Iterable[int], wrap=None) -> Iterable[int]:
         """
-        The modifier `inc:it` or `inc[wrap]:it` expects a sequence `it` of integers
-        (a binary string is interpreted as the sequence of its byte values), iterates it
-        cyclically and perpetually adds an increasing counter to the result. If `wrap`
-        is specified, then the counter is reduced modulo this number.
+        The modifier `inc:it` or `inc[N]:it` expects a sequence `it` of integers (a binary string
+        is interpreted as the sequence of its byte values), iterates it cyclically and perpetually
+        adds an increasing counter to the result. If the number `N` is specified, then the counter
+        is reduced modulo `N`.
         """
         def delay(_):
-            k = cycle(range(number(wrap))) if wrap else count()
-            for item in cycle(it):
+            k = itertools.cycle(range(number(wrap))) if wrap else itertools.count()
+            for item in itertools.cycle(it):
                 yield item + next(k)
         return delay
 
     @handler.register('dec')
     def dec(self, it: Iterable[int], wrap=None) -> Iterable[int]:
         """
-        Identical to `refinery.lib.argformats.DelayedNumSeqArgument.inc`, but the counter
-        is subtracted from `it`.
+        Identical to `refinery.lib.argformats.DelayedNumSeqArgument.inc`, but the counter is
+        subtracted from `it`.
         """
         def delay(_):
-            k = cycle(range(number(wrap))) if wrap else count()
-            for item in cycle(it):
+            k = itertools.cycle(range(number(wrap))) if wrap else itertools.count()
+            for item in itertools.cycle(it):
                 yield item - next(k)
         return delay
 
+    @handler.register('take')
+    def take(self, it: Iterable[int], bounds: Optional[str] = None):
+        """
+        The handler `take[start:stop:step]` expects an integer sequence as input and applies a slice
+        to it. Slices are given in Python syntax, so `take[::2]` will extract every second item from
+        the incoming data.
+        """
+        bounds = bounds and sliceobj(bounds) or slice(1, None)
+        try:
+            return it[bounds]
+        except TypeError:
+            subsequence = itertools.islice(it, bounds.start, bounds.stop, bounds.step)
+            if bounds.stop is not None:
+                subsequence = list(subsequence)
+                if all(t in range(0x100) for t in subsequence):
+                    subsequence = bytearray(subsequence)
+            return subsequence
+
+    @handler.register('cycle')
+    def cycle(self, it: Iterable[int]) -> Iterable[int]:
+        """
+        The `cycle` handler turns a finite integer sequence into an infinitely repeating integer sequence.
+        """
+        if isinstance(it, itertools.cycle):
+            return it
+        return itertools.cycle(it)
+
+    @handler.register('accu', final=True)
+    def accu(self, update: str, seed: Optional[str] = None) -> Iterable[int]:
+        """
+        The final handler `accu[seed=0]:update` expects `seed` and `expression` to be Python expressions
+        and generates an infinite integer sequence. The first element of this sequence is `seed`, which
+        is zero by default. Each subsequent element is the result of evaluating the `update` expression.
+        The `update` expression can use the special variable `A` to access the previous element of the
+        sequence. This handler can be used to implement simple finite state machines such as linear
+        congruency generators, for example.
+        """
+        update = PythonExpression(update, all_variables_allowed=True)
+        seed = seed and PythonExpression(seed, all_variables_allowed=True)
+
+        def finalize(data: Optional[Chunk] = None):
+            @lru_cache(maxsize=512, typed=False)
+            def accumulate(A): return update(A=A, **args)
+            args = data and data.meta or {}
+            A = seed and seed(**args) or 0
+            while True:
+                yield A
+                A = accumulate(A)
+
+        try:
+            return finalize()
+        except VariableMissing:
+            return finalize
+
+    @handler.register('be')
+    def be(self, arg: Union[int, ByteString]) -> int:
+        """
+        Convert a binary input into the integer that it encodes in big endian format, and vice versa.
+        """
+        if isinstance(arg, int):
+            size, remainder = divmod(arg.bit_length(), 8)
+            if remainder: size += 1
+            return arg.to_bytes(size, 'big')
+        return int.from_bytes(arg, 'big')
+
+    @handler.register('le')
+    def le(self, arg: Union[int, ByteString]) -> int:
+        """
+        Convert a binary input into the integer that it encodes in little endian format, and vice versa.
+        """
+        if isinstance(arg, int):
+            size, remainder = divmod(arg.bit_length(), 8)
+            if remainder: size += 1
+            return arg.to_bytes(size, 'little')
+        return int.from_bytes(arg, 'little')
+
+    @handler.register('reduce')
+    def reduce(self, it: Iterable[int], reduction: str, seed: Optional[str] = None) -> int:
+        """
+        The handler `reduce[reduction, seed=0]` has two parameters. The string `reduction` is a
+        Python expression that involves the two special variables `S` (the state) and `B`
+        (the current block value). This expression is evaluated for every `B` in the incoming
+        integer sequence and assigned back to `S`. The starting value of `S` is given by `seed`,
+        which has a default value of `0` and must also be given as a Python expression.
+        """
+        seed = seed and PythonExpression(seed, all_variables_allowed=True)
+        reduction = PythonExpression(reduction, all_variables_allowed=True)
+
+        def finalize(data: Optional[Chunk] = None):
+            args = data and data.meta or {}
+            return reduce(lambda S, B: reduction(S=S, B=B, **args), it, seed and seed(**args) or 0)
+
+        try:
+            return finalize()
+        except VariableMissing:
+            return finalize
+
+
+class DelayedBinaryArgument(DelayedArgument):
+
+    def __call__(self, data: Optional[ByteString] = None) -> bytes:
+        value = super().__call__(data=data)
+        if not isbuffer(value):
+            raise ArgumentTypeError(
+                F'The expression {self.expression} returned a value of type {type(value).__name__}, '
+                R'which could not be converted to a byte string.'
+            )
+        return value
+
+
+class DelayedNumSeqArgument(DelayedArgument):
+    """
+    A parser for sequences of numeric arguments. As `refinery.lib.argformats.DelayedNumSeqArgument.handler`
+    uses `refinery.lib.argformats.multibin`, it is possible to use any handler specified in
+    `refinery.lib.argformats.DelayedBinaryArgument` as long as these handlers precede any of the handlers
+    defined here.
+    """
+
+    def default_handler(self, expression: str) -> Iterable[int]:
+        """
+        Attempts to parse the input expression as a sequence of integers. If this fails, the handler defaults
+        to the parent `refinery.lib.argformats.DelayedArgument.default_handler`.
+        """
+        try:
+            return LazyPythonExpression(expression)
+        except Exception:
+            if isinstance(expression, str):
+                return super().default_handler(expression)
+            return expression
+
+    def __call__(self, data: Optional[Union[ByteString, Chunk]] = None) -> Iterable[int]:
+        value = super().__call__(data)
+        if hasattr(value, '__iter__'):
+            try:
+                if len(value) == 1:
+                    return RepeatedInteger(next(iter(value)))
+            except TypeError:
+                def rewind():
+                    yield top
+                    yield from it
+                it = iter(value)
+                top = next(it)
+                if not isinstance(top, int):
+                    raise ArgumentTypeError(F'The first item {top!r} of the iterable computed from {self.expression} was not an integer.')
+                return rewind()
+            else:
+                if not all(isinstance(t, int) for t in value):
+                    raise ArgumentTypeError(F'Not all elements of {value!r}, computed from {self.expression}, are integers.')
+                return value
+        if isinstance(value, int):
+            return RepeatedInteger(value)
+        raise ArgumentTypeError(
+            F'The value {value} computed from {self.expression} is of type {type(value).__name__} but the unit requested an '
+            R'integer or a sequence of integers.'
+        )
+
 
 class DelayedRegexpArgument(DelayedArgument):
     """
     A parser for regular expressions arguments.
     """
 
-    @DelayedArgumentDispatch
+    @DelayedArgumentDispatch.Inherit(DelayedArgument)
     def handler(self, expression: str) -> bytes:
         """
         The default handler encodes the input expression as latin-1 to return a binary
@@ -774,55 +947,27 @@ class DelayedNumberArgument(DelayedArgument):
     `refinery.lib.argformats.DelayedBinaryArgument` as long as these handlers precede any of
     the handlers defined here.
     """
-    def __init__(self, expression: str, min: int, max: int, bin: bool = False):
+    def __init__(self, expression: str, min: int, max: int):
         self.min = min
         self.max = max
-        self.bin = bin
         super().__init__(expression)
 
-    def _mbin(self, converter: DelayedType[int], expr: str) -> MaybeDelayedType[int]:
-        def bound_checked_converter(b: ByteString):
-            value = converter(b)
-            if self.min is not None and value < self.min or self.max is not None and value > self.max:
-                a = '-∞' if self.min is None else self.min
-                b = '∞' if self.max is None else self.max
-                raise ValueError(F'value {value} is out of bounds [{a}, {b}]')
-            return value
-        binary = multibin(expr)
-        if not binary:
-            raise ArgumentTypeError('received empty binary argument')
-        if callable(binary):
-            return lambda d: bound_checked_converter(binary(d))
-        return bound_checked_converter(binary)
+    def __call__(self, data: Union[ByteString, Chunk, None] = None) -> int:
+        value = super().__call__(data)
+        if not isinstance(value, int):
+            tv = type(value).__name__
+            raise ArgumentTypeError(F'The value {value} computed from {self.expression} is of type {tv}, it should be an integer.')
+        if self.min is not None and value < self.min or self.max is not None and value > self.max:
+            a = '-∞' if self.min is None else self.min
+            b = '∞' if self.max is None else self.max
+            raise ArgumentTypeError(F'value {value} is out of bounds [{a}, {b}]')
+        return value
 
-    @DelayedArgumentDispatch
-    def handler(self, expression: str) -> int:
+    def default_handler(self, expression: str) -> int:
         """
         The default handler: Attempts to parse the input expression as an integer.
         """
-        def parse(b: Union[int, ByteString]) -> int:
-            if isinstance(b, int):
-                return b
-            expression = PythonExpression(b.decode('utf8'))
-            t = expression()
-            if self.bin and not isinstance(t, int):
-                raise ValueError(F'the expression with value {expression} is not an integer')
-            return t
-        return self._mbin(parse, expression)
-
-    @handler.register('be', final=True)
-    def be(self, expression: str) -> int:
-        def be(b: ByteString): return int.from_bytes(b, 'big')
-        return self._mbin(be, expression)
-
-    @handler.register('le', final=True)
-    def le(self, expression: str) -> int:
-        def le(b: ByteString): return int.from_bytes(b, 'little')
-        return self._mbin(le, expression)
-
-    @handler.register('h', 'H', final=True)
-    def hex(self, expression: str) -> int:
-        return int(expression.upper().rstrip('H'), 16)
+        return LazyPythonExpression(expression)
 
 
 class number:
@@ -840,9 +985,10 @@ class number:
             return value
         try:
             delay = DelayedNumberArgument(value, self.min, self.max)
-            return delay()
-        except TooLazy:
-            return delay
+            try:
+                return delay()
+            except TooLazy:
+                return delay
         except ParserError:
             import re
             match = re.fullmatch('(?:0x)?([A-F0-9]+)H?', value, flags=re.IGNORECASE)
@@ -860,16 +1006,6 @@ expressions.
 """
 
 
-def numbin(expression: Union[int, str]) -> Union[ByteString, int, DelayedNumberArgument]:
-    """
-    An argument parser using `refinery.lib.argformats.DelayedNumberArgument` but also allowing
-    binary arguments. The result is either a byte string or a number.
-    """
-    arg = DelayedNumberArgument(expression, None, None, True)
-    with suppress(TooLazy): return arg()
-    return arg
-
-
 def numseq(expression: Union[int, str]) -> Union[Iterable[int], DelayedNumSeqArgument]:
     """
     This is the argument parser type that uses `refinery.lib.argformats.DelayedNumSeqArgument`.
@@ -882,6 +1018,18 @@ def numseq(expression: Union[int, str]) -> Union[Iterable[int], DelayedNumSeqArg
     return arg
 
 
+def multibin(expression: Union[str, bytes, bytearray]) -> Union[bytes, DelayedArgument]:
+    """
+    This is the argument parser type that uses `refinery.lib.argformats.DelayedBinaryArgument`.
+    """
+    if not isinstance(expression, str):
+        return bytes(expression)
+    arg = DelayedBinaryArgument(expression)
+    with suppress(TooLazy):
+        return arg()
+    return arg
+
+
 def regexp(expression: str) -> Union[int, bytes, DelayedRegexpArgument]:
     """
     This is the argument parser type that uses `refinery.lib.argformats.DelayedRegexpArgument`.
