@@ -15,6 +15,7 @@ tlds = ['xn\\-\\-vermgensberatung\\-pwb',
  'travelersinsurance',
  'xn\\-\\-fzys8d69uvgm',
  'xn\\-\\-mgba7c0bbn0a',
+ 'xn\\-\\-mgbcpq6gpa1a',
  'xn\\-\\-xkc2al3hye2a',
  'xn\\-\\-i1b6b1a6a2e',
  'xn\\-\\-kcrx77d1x4a',
@@ -29,6 +30,7 @@ tlds = ['xn\\-\\-vermgensberatung\\-pwb',
  'xn\\-\\-b4w605ferd',
  'xn\\-\\-fiq228c5hs',
  'xn\\-\\-h2breg3eve',
+ 'xn\\-\\-jlq480n2rg',
  'xn\\-\\-jlq61u9w7b',
  'xn\\-\\-mgba3a3ejt',
  'xn\\-\\-mgbaam7a8h',
@@ -41,6 +43,7 @@ tlds = ['xn\\-\\-vermgensberatung\\-pwb',
  'americanexpress',
  'kerryproperties',
  'sandvikcoromant',
+ 'xn\\-\\-cckwcxetd',
  'xn\\-\\-eckvdtc9d',
  'xn\\-\\-fpcrj9c3d',
  'xn\\-\\-fzc2c9e2c',
@@ -151,6 +154,7 @@ tlds = ['xn\\-\\-vermgensberatung\\-pwb',
  'xn\\-\\-mxtq1m',
  'xn\\-\\-o3cw4h',
  'xn\\-\\-pssy2u',
+ 'xn\\-\\-q7ce6a',
  'xn\\-\\-unup4y',
  'xn\\-\\-wgbh1c',
  'xn\\-\\-wgbl6a',
@@ -510,6 +514,7 @@ tlds = ['xn\\-\\-vermgensberatung\\-pwb',
  'alipay',
  'alsace',
  'alstom',
+ 'amazon',
  'anquan',
  'aramco',
  'author',
@@ -1235,6 +1240,7 @@ tlds = ['xn\\-\\-vermgensberatung\\-pwb',
  'ski',
  'sky',
  'soy',
+ 'spa',
  'srl',
  'stc',
  'tab',
