@@ -105,6 +105,8 @@ the following Python code:
     In [2]: B'OOOOOOOO' | chop(2) [ ccp(B'F') | cca(B'.') ]
     Out[2]: bytearray(b'FOO.FOO.FOO.FOO.')
 """
+from __future__ import annotations
+
 import sys
 import os
 import inspect
@@ -112,9 +114,9 @@ import inspect
 from enum import IntEnum, Enum
 from functools import wraps
 from collections import OrderedDict
-from typing import Iterable, BinaryIO, Type, Union, List, Optional, Callable, Tuple, Any, ByteString, no_type_check, get_type_hints
+from typing import Iterable, BinaryIO, Type, TypeVar, Union, List, Optional, Callable, Tuple, Any, ByteString, no_type_check, get_type_hints
 from argparse import (
-    Namespace,
+    ArgumentTypeError, Namespace,
     ONE_OR_MORE,
     OPTIONAL,
     REMAINDER,
@@ -516,11 +518,13 @@ class ArgumentSpecification(OrderedDict):
         self[dest] = argument
 
 
-def UnitProcessorBoilerplate(
-    operation: Callable[[Any, ByteString], Any]
-) -> Callable[[ByteString], Any]:
+DataType = TypeVar('DataType', bound=ByteString)
+ProcType = Callable[['Unit', ByteString], Optional[Union[DataType, Iterable[DataType]]]]
+
+
+def UnitProcessorBoilerplate(operation: ProcType[ByteString]) -> ProcType[Chunk]:
     @wraps(operation)
-    def wrapped(self, data: ByteString) -> Union[Optional[ByteString], Iterable[ByteString]]:
+    def wrapped(self, data: ByteString) -> Optional[Union[Chunk, Iterable[Chunk]]]:
         ChunkType = Chunk
         if data is None:
             data = B''
@@ -902,19 +906,22 @@ class Unit(metaclass=Executable, abstract=True):
 
     def _exception_handler(self, exception: BaseException):
         if self.log_level <= LogLevel.DETACHED:
-            raise
+            if isinstance(exception, RefineryPartialResult) and self.args.lenient:
+                return None
+            raise exception
         elif isinstance(exception, RefineryCriticalException):
             self.log_warn(F'critical error, terminating: {exception}')
-            raise
+            raise exception
         elif isinstance(exception, VariableMissing):
             self.log_warn('critical error:', exception)
             raise RefineryCriticalException
         elif isinstance(exception, GeneratorExit):
-            raise
+            raise exception
         elif isinstance(exception, RefineryPartialResult):
-            if not self.log_level:
+            warning_enabled = self.log_warn(F'error, partial result returned: {exception}')
+            if not self.args.lenient:
                 return None
-            elif not self.log_warn(F'error, partial result returned: {exception}'):
+            if not warning_enabled:
                 raise exception
             return exception.partial
         else:
@@ -938,7 +945,7 @@ class Unit(metaclass=Executable, abstract=True):
         if self._framed:
             return self._framed
 
-        def normalized_action(data: ByteString) -> Iterable[ByteString]:
+        def normalized_action(data: ByteString) -> Iterable[Chunk]:
             try:
                 result = self.act(data)
                 if inspect.isgenerator(result):
@@ -948,7 +955,7 @@ class Unit(metaclass=Executable, abstract=True):
             except BaseException as B:
                 result = self._exception_handler(B)
                 if result is not None:
-                    yield result
+                    yield self.labelled(result, error=str(B))
 
         self._framed = Framed(
             normalized_action,
@@ -987,7 +994,7 @@ class Unit(metaclass=Executable, abstract=True):
         self._source = stream
 
     @property
-    def nozzle(self) -> 'Unit':
+    def nozzle(self) -> Unit:
         """
         The nozzle is defined recursively as the nozzle of `refinery.units.Unit.source`
         and `self` if no such thing exists. In other words, it is the leftmost unit in
@@ -998,7 +1005,7 @@ class Unit(metaclass=Executable, abstract=True):
         except AttributeError:
             return self
 
-    def __getitem__(self, unit: Union['Unit', Type['Unit'], slice]):
+    def __getitem__(self, unit: Union[Unit, Type[Unit], slice]):
         if isinstance(unit, type):
             unit = unit()
         alpha = self.__copy__()
@@ -1024,7 +1031,7 @@ class Unit(metaclass=Executable, abstract=True):
             return self
         return self(stream)
 
-    def __or__(self, stream: Union[BinaryIO, 'Unit']):
+    def __or__(self, stream: Union[BinaryIO, Unit]):
         try:
             if isinstance(stream, type):
                 stream = stream()
@@ -1108,11 +1115,11 @@ class Unit(metaclass=Executable, abstract=True):
         except StopIteration:
             return B''
 
-    def act(self, data: Union[Chunk, ByteString]) -> Union[ByteString, Chunk, None]:
+    def act(self, data: Union[Chunk, ByteString]) -> Optional[Chunk]:
         op = self.reverse if self.args.reverse else self.process
         return op(self.args @ data)
 
-    def __call__(self, data: Union[ByteString, Chunk, None] = None) -> bytes:
+    def __call__(self, data: Optional[Union[ByteString, Chunk]] = None) -> bytes:
         with MemoryFile(data) if data else open(os.devnull, 'rb') as stdin:
             with MemoryFile() as stdout:
                 return (stdin | self | stdout).getvalue()
@@ -1210,21 +1217,18 @@ class Unit(metaclass=Executable, abstract=True):
         """
         base = argp.add_argument_group('generic options')
 
-        base.add_argument('-h', '--help', action='help', help='Show this help message and exit.')
         base.set_defaults(reverse=False, squeeze=False)
-
-        if cls.is_reversible:
-            base.add_argument('-R', '--reverse', action='store_true', help='Use the reverse operation.')
-
-        if cls.is_multiplex:
-            base.add_argument('-Z', '--squeeze', action='store_true', help=SUPPRESS)
-
+        base.add_argument('-h', '--help', action='help', help='Show this help message and exit.')
+        base.add_argument('-L', '--lenient', action='store_true', help='Allow partial results as output.')
         base.add_argument('-Q', '--quiet', action='store_true', help='Disables all log output.')
         base.add_argument('-0', '--devnull', action='store_true', help='Do not produce any output.')
         base.add_argument('-v', '--verbose', action='count', default=LogLevel.WARN,
             help='Specify up to two times to increase log level.')
         argp.add_argument('--debug-timing', dest='dtiming', action='store_true', help=SUPPRESS)
 
+        if cls.is_reversible:
+            base.add_argument('-R', '--reverse', action='store_true', help='Use the reverse operation.')
+
         groups = {None: argp}
 
         for argument in reversed(cls._argspec_.values()):
@@ -1294,6 +1298,7 @@ class Unit(metaclass=Executable, abstract=True):
         else:
             unit.args._store(_argo=argp.order)
             unit.args.quiet = args.quiet
+            unit.args.lenient = args.lenient
 
             unit.args.squeeze = args.squeeze
             unit.args.dtiming = args.dtiming
@@ -1364,6 +1369,7 @@ class Unit(metaclass=Executable, abstract=True):
                 unit = cls.assemble(*argv)
             except ArgparseError as ap:
                 ap.parser.error_commandline(str(ap))
+                return
             except Exception as msg:
                 import traceback
                 cls._output('initialization failed:', msg)
@@ -1392,6 +1398,8 @@ class Unit(metaclass=Executable, abstract=True):
             try:
                 with open(os.devnull, 'wb') if unit.args.devnull else sys.stdout.buffer as output:
                     source | unit | output
+            except ArgumentTypeError as E:
+                unit.output('delayed argument initialization failed:', str(E))
             except KeyboardInterrupt:
                 unit.output('aborting due to keyboard interrupt')
             except OSError:
